//==============================================================
// Copyright  2020 Intel Corporation
//
// SPDX-License-Identifier: MIT
// =============================================================

#include <CL/sycl.hpp>

#include <ctime>
#include <cstdlib>
#include <iomanip>
#include <iostream>

using namespace std;
using namespace cl::sycl;

constexpr std::uint32_t N = 20;

int main(int argc, char** argv)
{
	std::cout << "\nA Parallel f = Pow(v,r) in C++17 (CL/SYCL) by Arthur V. Ratz @ CodeProject.Com" << "\n";
	std::cout << "==============================================================================" << "\n\n";

	cl::sycl::queue q{};

	double f = 1.0f; double v = .0f; std::uint32_t r = 0;

	std::srand(std::time(nullptr));

	std::uint32_t n = N;

	while (--n != 0xFFFFFFFF)
	{
		r = std::rand() % 5 + 2;
		v = std::rand() / (double)RAND_MAX + std::rand() % 10;

		std::cout << v << "^" << r << " = ";

		cl::sycl::buffer<double, 1> fb{ &f, 1L };
		cl::sycl::buffer<double, 1> vb{ &v, 1L };

		q.submit([&](cl::sycl::handler& cgh) {
			auto fa = fb.get_access<cl::sycl::access::mode::read_write>(cgh);
			auto va = vb.get_access<cl::sycl::access::mode::read_write>(cgh);
			cgh.parallel_for(cl::sycl::range<1>{r}, \
				[=](cl::sycl::id<1> idx) { fa[0] *= va[0]; });
			});

		q.wait_and_throw();

		fb.get_access<cl::sycl::access::mode::read>();

		std::cout << std::setprecision(10) << f << "\n\n";
	}

	return 0;
}